@@ -1,9 +1,8 @@
-use ark_bls12_381::Fr as BlsFr;
-use ark_bn254::Fr as Bn254Fr;
 use ark_std::rand::rngs::OsRng;
-use arkworks_circuits::setup::mixer::{setup_leaf_with_privates_raw_x5_5, setup_leaf_x5_5};
-use arkworks_gadgets::prelude::*;
-use arkworks_utils::utils::common::Curve as ArkworksCurve;
+use arkworks_setups::common::Leaf;
+
+use crate::{MixerR1CSProverBls381_30, MixerR1CSProverBn254_30};
+use arkworks_setups::{Curve as ArkCurve, MixerProver};
 
 use crate::types::{Curve, OpStatusCode, OperationError};
 
@@ -12,13 +11,13 @@ pub fn generate_secrets(
 	width: usize,
 	curve: Curve,
 	rng: &mut OsRng,
-) -> Result<Vec<u8>, OperationError> {
-	let (secret_bytes, nullifier_bytes, ..) = match (curve, exponentiation, width) {
-		(Curve::Bls381, 5, 5) => setup_leaf_x5_5::<BlsFr, _>(ArkworksCurve::Bls381, rng),
-		(Curve::Bn254, 5, 5) => setup_leaf_x5_5::<Bn254Fr, _>(ArkworksCurve::Bn254, rng),
+) -> Result<[Vec<u8>; 2], OperationError> {
+	let sec = match (curve, exponentiation, width) {
+		(Curve::Bls381, 5, 3) => MixerR1CSProverBls381_30::create_random_leaf(ArkCurve::Bls381, rng),
+		(Curve::Bn254, 5, 3) => MixerR1CSProverBn254_30::create_random_leaf(ArkCurve::Bn254, rng),
 		_ => {
 			let message = format!(
-				"No Mixer secrets setup available for curve {}, exponentiation {} , and width {}",
+				"No Mixer secrets setup available for curve {}, exponentiation {}, and width {}",
 				curve, exponentiation, width
 			);
 			return Err(OperationError::new_with_message(OpStatusCode::SecretGenFailed, message));
@@ -26,16 +25,17 @@ pub fn generate_secrets(
 	}
 	.map_err(|e| OperationError::new_with_message(OpStatusCode::SecretGenFailed, e.to_string()))?;
 
-	let secrets = [secret_bytes, nullifier_bytes].concat();
+	let secrets = [sec.secret_bytes, sec.nullifier_bytes];
 
 	Ok(secrets)
 }
+
 pub fn get_leaf_with_private_raw(
 	curve: Curve,
 	width: usize,
 	exponentiation: i8,
 	raw: &[u8],
-) -> Result<(Vec<u8>, Vec<u8>), OperationError> {
+) -> Result<Leaf, OperationError> {
 	if raw.len() < 64 {
 		return Err(OpStatusCode::InvalidNoteSecrets.into());
 	}
@@ -44,11 +44,13 @@ pub fn get_leaf_with_private_raw(
 	let nullifer = raw[32..64].to_vec();
 	// (leaf_bytes, nullifier_hash_bytes)
 	let sec = match (curve, exponentiation, width) {
-		(Curve::Bls381, 5, 5) => setup_leaf_with_privates_raw_x5_5::<BlsFr>(ArkworksCurve::Bls381, secrets, nullifer),
-		(Curve::Bn254, 5, 5) => setup_leaf_with_privates_raw_x5_5::<Bn254Fr>(ArkworksCurve::Bn254, secrets, nullifer),
+		(Curve::Bls381, 5, 3) => {
+			MixerR1CSProverBls381_30::create_leaf_with_privates(ArkCurve::Bls381, secrets, nullifer)
+		}
+		(Curve::Bn254, 5, 3) => MixerR1CSProverBn254_30::create_leaf_with_privates(ArkCurve::Bn254, secrets, nullifer),
 		_ => {
 			let message = format!(
-				"No Mixer leaf setup for curve {}, exponentiation {} , and width {}",
+				"No Mixer leaf setup for curve {}, exponentiation {}, and width {}",
 				curve, exponentiation, width
 			);
 			return Err(OperationError::new_with_message(
