@@ -1,11 +1,24 @@
-use std::convert::{TryFrom, TryInto};
+#![allow(clippy::unused_unit)]
+
+use core::convert::{TryFrom, TryInto};
+
+use ark_ff::{BigInteger, PrimeField};
 
 use js_sys::{Array, JsString, Uint8Array};
 use rand::rngs::OsRng;
 use wasm_bindgen::prelude::*;
 
+use ark_bls12_381::Bls12_381;
+use ark_bn254::{Bn254, Fr as Bn254Fr};
+use arkworks_native_gadgets::merkle_tree::SparseMerkleTree;
+use arkworks_native_gadgets::poseidon::Poseidon;
+use arkworks_setups::common::{setup_params, setup_tree_and_create_path, verify_unchecked_raw};
+use arkworks_setups::Curve as ArkCurve;
+use wasm_bindgen::__rt::std::collections::btree_map::BTreeMap;
+
 use crate::note::JsNote;
-use crate::types::{Backend, Curve, Leaves, NotePrefix, OpStatusCode, OperationError, Uint8Arrayx32};
+use crate::types::{Backend, Curve, Leaves, NoteProtocol, OpStatusCode, OperationError, Uint8Arrayx32, WasmCurve};
+use crate::TREE_HEIGHT;
 
 mod anchor;
 mod mixer;
@@ -18,6 +31,7 @@ pub fn truncate_and_pad(t: &[u8]) -> Vec<u8> {
 	truncated_bytes
 }
 
+#[allow(clippy::unused_unit)]
 #[wasm_bindgen]
 #[derive(Debug, Eq, PartialEq)]
 pub struct Proof {
@@ -35,6 +49,7 @@ pub struct Proof {
 	pub leaf: Vec<u8>,
 }
 
+#[allow(clippy::unused_unit)]
 #[wasm_bindgen]
 impl Proof {
 	#[wasm_bindgen(getter)]
@@ -55,6 +70,12 @@ impl Proof {
 		let root = hex::encode(&self.root);
 		root.into()
 	}
+
+	#[wasm_bindgen(getter)]
+	pub fn roots(&self) -> Array {
+		let roots: Array = self.roots.iter().map(|i| JsString::from(hex::encode(i))).collect();
+		roots
+	}
 }
 #[derive(Debug, Eq, PartialEq, Clone)]
 pub struct MixerProofInput {
@@ -62,7 +83,7 @@ pub struct MixerProofInput {
 	pub width: usize,
 	pub curve: Curve,
 	pub backend: Backend,
-	pub secrets: Vec<u8>,
+	pub secret: Vec<u8>,
 	pub nullifier: Vec<u8>,
 	pub recipient: Vec<u8>,
 	pub relayer: Vec<u8>,
@@ -79,21 +100,21 @@ pub struct AnchorProofInput {
 	pub width: usize,
 	pub curve: Curve,
 	pub backend: Backend,
-	pub secrets: Vec<u8>,
+	pub secret: Vec<u8>,
 	pub nullifier: Vec<u8>,
 	pub recipient: Vec<u8>,
 	pub relayer: Vec<u8>,
 	pub pk: Vec<u8>,
 	pub refund: u128,
 	pub fee: u128,
-	pub chain_id: u128,
+	pub chain_id: u64,
 	pub leaves: Vec<Vec<u8>>,
 	pub leaf_index: u64,
 	/// get roots for linkable tree
 	pub roots: Vec<Vec<u8>>,
 	/// EMPTY commitment if withdrawing [0u8;32]
 	/// not EMPTY if refreshing
-	pub commitment: [u8; 32],
+	pub refresh_commitment: [u8; 32],
 }
 
 #[derive(Debug, Eq, PartialEq, Clone)]
@@ -109,7 +130,7 @@ impl ProofInput {
 			ProofInput::Anchor(_) => {
 				let message = "Can't construct proof input for AnchorProofInput from mixer input".to_string();
 				Err(OperationError::new_with_message(
-					OpStatusCode::InvalidNotePrefix,
+					OpStatusCode::InvalidNoteProtocol,
 					message,
 				))
 			}
@@ -122,7 +143,7 @@ impl ProofInput {
 			ProofInput::Mixer(_) => {
 				let message = "Can't cant construct proof input for MixerProofInput from anchor input ".to_string();
 				Err(OperationError::new_with_message(
-					OpStatusCode::InvalidNotePrefix,
+					OpStatusCode::InvalidNoteProtocol,
 					message,
 				))
 			}
@@ -130,13 +151,21 @@ impl ProofInput {
 	}
 }
 
+#[allow(unused_macros)]
+macro_rules! console_log {
+	// Note that this is using the `log` function imported above during
+	// `bare_bones`
+	($($t:tt)*) => (crate::types::log(&format_args!($($t)*).to_string()))
+}
+
 #[wasm_bindgen]
+#[derive(Debug, Clone)]
 pub struct JsProofInput {
 	#[wasm_bindgen(skip)]
 	pub inner: ProofInput,
 }
 #[wasm_bindgen]
-#[derive(Debug, Eq, PartialEq)]
+#[derive(Debug, Eq, PartialEq, Default)]
 pub struct ProofInputBuilder {
 	#[wasm_bindgen(skip)]
 	pub recipient: Option<Vec<u8>>,
@@ -156,7 +185,7 @@ pub struct ProofInputBuilder {
 	pub note: Option<JsNote>,
 	#[wasm_bindgen(skip)]
 	/// required only for [anchor,]
-	pub commitment: Option<[u8; 32]>,
+	pub refresh_commitment: Option<[u8; 32]>,
 	#[wasm_bindgen(skip)]
 	/// required only for [anchor,]
 	pub roots: Option<Vec<Vec<u8>>>,
@@ -179,28 +208,33 @@ impl ProofInputBuilder {
 
 		let fee = self.fee.ok_or(OpStatusCode::InvalidFee)?;
 		let refund = self.refund.ok_or(OpStatusCode::InvalidRefund)?;
-
-		let target_chain_id = note
-			.target_chain_id
-			.parse()
-			.map_err(|_| OpStatusCode::InvalidTargetChain)?;
-		let proof_target = note.prefix;
+		let proof_target = note.protocol;
 		let width = note.width;
 		let exponentiation = note.exponentiation;
 		let backend = note.backend;
 		let curve = note.curve;
-		let note_secrets = note.secret;
-		let secrets = note_secrets[..32].to_vec();
-		let nullifier = note_secrets[32..64].to_vec();
-
+		let note_secrets = note.secrets;
 		let processed_relayer = truncate_and_pad(&relayer);
 		let processed_recipient = truncate_and_pad(&recipient);
 		match proof_target {
-			NotePrefix::Mixer => {
+			NoteProtocol::Mixer => {
+				// Mixer secrets are structures as a vector of [secret, nullifier] or
+				// concatenated bytes
+				let mut secret = Vec::new();
+				let mut nullifier = Vec::new();
+				if note_secrets.len() == 1 && note_secrets[0].len() >= 64 {
+					secret.extend_from_slice(&note_secrets[0][0..32]);
+					nullifier.extend_from_slice(&note_secrets[0][32..64]);
+				} else {
+					secret = note_secrets[0].clone();
+					nullifier = note_secrets[1].clone();
+				}
+
 				let mixer_proof_input = MixerProofInput {
-					exponentiation,
-					width,
-					curve,
+					exponentiation: exponentiation.unwrap_or(5),
+					width: width.unwrap_or(3),
+					curve: curve.unwrap_or(Curve::Bn254),
+					backend: backend.unwrap_or(Backend::Circom),
 					pk,
 					recipient: processed_recipient,
 					relayer: processed_relayer,
@@ -208,59 +242,128 @@ impl ProofInputBuilder {
 					fee,
 					leaf_index,
 					leaves,
-					secrets,
+					secret,
 					nullifier,
-					backend,
 					chain_id: 0,
 				};
 				Ok(ProofInput::Mixer(mixer_proof_input))
 			}
-			NotePrefix::Anchor => {
-				let commitment = self.commitment.ok_or(OpStatusCode::CommitmentNotSet)?;
+			NoteProtocol::Anchor => {
+				// Mixer secrets are structures as a vector of [secret, nullifier] or
+				// concatenated bytes
+				let chain_id = note
+					.target_chain_id
+					.parse()
+					.map_err(|_| OpStatusCode::InvalidTargetChain)?;
+				let mut secret = Vec::new();
+				let mut nullifier = Vec::new();
+				if note_secrets.len() == 1 && note_secrets[0].len() >= 64 {
+					secret.extend_from_slice(&note_secrets[0][0..32]);
+					nullifier.extend_from_slice(&note_secrets[0][32..64]);
+				} else {
+					// Anchor note secrets are structure as a vector of [chain_id, secret,
+					// nullifier]
+					nullifier = note_secrets[1].clone();
+					secret = note_secrets[2].clone();
+				}
+
+				let refresh_commitment = self.refresh_commitment.ok_or(OpStatusCode::CommitmentNotSet)?;
 				let roots = self.roots.ok_or(OpStatusCode::RootsNotSet)?;
 
 				let anchor_input = AnchorProofInput {
-					exponentiation,
-					width,
-					curve,
-					backend,
-					secrets,
+					exponentiation: exponentiation.unwrap_or(5),
+					width: width.unwrap_or(4),
+					curve: curve.unwrap_or(Curve::Bn254),
+					backend: backend.unwrap_or(Backend::Circom),
+					secret,
 					nullifier,
 					recipient: processed_recipient,
 					relayer: processed_relayer,
 					pk,
 					refund,
 					fee,
-					chain_id: target_chain_id,
+					chain_id,
 					leaves,
 					leaf_index,
 					roots,
-					commitment,
+					refresh_commitment,
 				};
 				Ok(ProofInput::Anchor(anchor_input))
 			}
-			_ => Err(OpStatusCode::InvalidNotePrefix),
+			_ => Err(OpStatusCode::InvalidNoteProtocol),
 		}
 	}
 }
 
-impl Default for ProofInputBuilder {
-	fn default() -> Self {
-		Self {
-			recipient: None,
-			relayer: None,
-			leaves: None,
-			leaf_index: None,
-			fee: None,
-			refund: None,
-			pk: None,
-			note: None,
-			commitment: None,
-			roots: None,
-		}
-	}
+#[wasm_bindgen]
+pub struct AnchorMTBn254X5 {
+	#[wasm_bindgen(skip)]
+	pub inner: SparseMerkleTree<Bn254Fr, Poseidon<Bn254Fr>, TREE_HEIGHT>,
 }
 
+#[allow(clippy::unused_unit)]
+#[wasm_bindgen]
+impl AnchorMTBn254X5 {
+	#[wasm_bindgen(constructor)]
+	pub fn new(initial_leaves: Leaves, leaf_index: JsString) -> Result<AnchorMTBn254X5, JsValue> {
+		let leaf_index: String = leaf_index.into();
+		let leaf_index: u64 = leaf_index.parse().expect("Failed to parse the leaf index");
+		let leaves: Vec<_> = Array::from(&initial_leaves)
+			.to_vec()
+			.into_iter()
+			.map(|v| Uint8Array::new_with_byte_offset_and_length(&v, 0, 32))
+			.map(Uint8Arrayx32::try_from)
+			.collect::<Result<Vec<_>, _>>()
+			.map_err(|_| OpStatusCode::InvalidLeaves)?
+			.into_iter()
+			.map(|v| Bn254Fr::from_le_bytes_mod_order(v.0.as_ref()))
+			.collect();
+
+		let curve = ArkCurve::Bn254;
+		let params3 = setup_params::<Bn254Fr>(curve, 5, 3);
+		let poseidon3 = Poseidon::new(params3);
+
+		let (tree, _) = setup_tree_and_create_path::<Bn254Fr, Poseidon<Bn254Fr>, TREE_HEIGHT>(
+			&poseidon3, &leaves, leaf_index, &[0u8; 32],
+		)
+		.unwrap();
+		Ok(Self { inner: tree })
+	}
+
+	#[wasm_bindgen(getter)]
+	pub fn root(&self) -> JsString {
+		let root = self.inner.root().into_repr().to_bytes_le().to_vec();
+		JsString::from(hex::encode(root))
+	}
+
+	#[wasm_bindgen]
+	pub fn insert(&mut self, leaves: Leaves) -> Result<(), JsValue> {
+		let mut leaves_bt = BTreeMap::<_, Bn254Fr>::new();
+		Array::from(&leaves)
+			.to_vec()
+			.into_iter()
+			.map(|v| Uint8Array::new_with_byte_offset_and_length(&v, 0, 32))
+			.map(Uint8Arrayx32::try_from)
+			.collect::<Result<Vec<_>, _>>()
+			.map_err(|_| OpStatusCode::InvalidLeaves)?
+			.iter()
+			.for_each(|leaf| {
+				leaves_bt
+					.insert(
+						leaves_bt.len() as u32 + 1_u32,
+						Bn254Fr::from_le_bytes_mod_order(leaf.0.as_ref()),
+					)
+					.unwrap();
+			});
+
+		let params3 = setup_params::<Bn254Fr>(ArkCurve::Bn254, 5, 3);
+		let poseidon3 = Poseidon::new(params3);
+		self.inner
+			.insert_batch(&leaves_bt, &poseidon3)
+			.map_err(|_| OpStatusCode::InvalidLeaves)?;
+		Ok(())
+	}
+}
 #[wasm_bindgen]
 impl ProofInputBuilder {
 	#[wasm_bindgen(constructor)]
@@ -284,12 +387,14 @@ impl ProofInputBuilder {
 		Ok(())
 	}
 
-	#[wasm_bindgen(js_name = setCommiment)]
-	pub fn set_commitment(&mut self, commitment: JsString) -> Result<(), JsValue> {
-		let c: String = commitment.into();
-		let commitment = hex::decode(c).map_err(|_| OpStatusCode::CommitmentNotSet)?;
-		let commitment: [u8; 32] = commitment.try_into().map_err(|_| OpStatusCode::CommitmentNotSet)?;
-		self.commitment = Some(commitment);
+	#[wasm_bindgen(js_name = setRefreshCommitment)]
+	pub fn set_refresh_commitment(&mut self, refresh_commitment: JsString) -> Result<(), JsValue> {
+		let c: String = refresh_commitment.into();
+		let refresh_commitment = hex::decode(c).map_err(|_| OpStatusCode::CommitmentNotSet)?;
+		let refresh_commitment: [u8; 32] = refresh_commitment
+			.try_into()
+			.map_err(|_| OpStatusCode::CommitmentNotSet)?;
+		self.refresh_commitment = Some(refresh_commitment);
 		Ok(())
 	}
 
@@ -383,19 +488,33 @@ pub fn generate_proof_js(proof_input: JsProofInput) -> Result<Proof, JsValue> {
 	}
 	.map_err(|e| e.into())
 }
+
+#[wasm_bindgen]
+pub fn validate_proof(proof: &Proof, vk: JsString, curve: WasmCurve) -> Result<bool, JsValue> {
+	let vk_string: String = vk.into();
+	let curve: String = JsValue::from(&curve).as_string().ok_or(OpStatusCode::InvalidCurve)?;
+	let curve: Curve = curve.parse().map_err(|_| OpStatusCode::InvalidCurve)?;
+
+	let vk = hex::decode(vk_string).expect("Field to deserialize");
+	let is_valid = match curve {
+		Curve::Bls381 => verify_unchecked_raw::<Bls12_381>(&proof.public_inputs, &vk, &proof.proof),
+		Curve::Bn254 => verify_unchecked_raw::<Bn254>(&proof.public_inputs, &vk, &proof.proof),
+	}
+	.map_err(|e| OperationError::new_with_message(OpStatusCode::InvalidProof, e.to_string()))?;
+
+	Ok(is_valid)
+}
 #[cfg(test)]
 mod test {
-	use arkworks_circuits::setup::common::verify_unchecked_raw;
+	use super::*;
 
 	use wasm_bindgen_test::*;
 
-	use super::*;
 	use crate::proof::test_utils::{
-		generate_anchor_test_setup, generate_mixer_test_setup, AnchorTestSetup, MixerTestSetup, ANCHOR_NOTE_X5_4,
-		DECODED_SUBSTRATE_ADDRESS, MIXER_NOTE_X5_5,
+		generate_anchor_test_setup, generate_mixer_test_setup, AnchorTestSetup, MixerTestSetup, ANCHOR_NOTE_V1_X5_4,
+		ANCHOR_NOTE_V2_X5_4, DECODED_SUBSTRATE_ADDRESS, MIXER_NOTE_V1_X5_5,
 	};
 
-	use arkworks_circuits::prelude::ark_bn254::Bn254;
 	const TREE_DEPTH: usize = 30;
 
 	#[wasm_bindgen_test]
@@ -406,7 +525,7 @@ mod test {
 			proof_input_builder,
 			leaf_bytes,
 			..
-		} = generate_mixer_test_setup(DECODED_SUBSTRATE_ADDRESS, DECODED_SUBSTRATE_ADDRESS, MIXER_NOTE_X5_5);
+		} = generate_mixer_test_setup(DECODED_SUBSTRATE_ADDRESS, DECODED_SUBSTRATE_ADDRESS, MIXER_NOTE_V1_X5_5);
 
 		let truncated_substrate_relayer_address = truncate_and_pad(&relayer);
 		let truncated_substrate_recipient_address = truncate_and_pad(&recipient);
@@ -439,7 +558,11 @@ mod test {
 			roots_raw,
 			leaf_bytes,
 			..
-		} = generate_anchor_test_setup(DECODED_SUBSTRATE_ADDRESS, DECODED_SUBSTRATE_ADDRESS, ANCHOR_NOTE_X5_4);
+		} = generate_anchor_test_setup(
+			DECODED_SUBSTRATE_ADDRESS,
+			DECODED_SUBSTRATE_ADDRESS,
+			ANCHOR_NOTE_V1_X5_4,
+		);
 		let anchor_input = proof_input_builder.build().unwrap().anchor_input().unwrap();
 		let truncated_substrate_relayer_address = truncate_and_pad(&relayer);
 		let truncated_substrate_recipient_address = truncate_and_pad(&recipient);
@@ -452,7 +575,7 @@ mod test {
 			hex::encode(&truncated_substrate_relayer_address)
 		);
 
-		assert_eq!(hex::encode(anchor_input.commitment), hex::encode([0u8; 32]));
+		assert_eq!(hex::encode(anchor_input.refresh_commitment), hex::encode([0u8; 32]));
 		assert_eq!(hex::encode(&anchor_input.roots[0]), hex::encode(&roots_raw[0]));
 		assert_eq!(anchor_input.roots.len(), roots_raw.len());
 
@@ -469,7 +592,7 @@ mod test {
 			proof_input_builder,
 			vk,
 			..
-		} = generate_mixer_test_setup(DECODED_SUBSTRATE_ADDRESS, DECODED_SUBSTRATE_ADDRESS, MIXER_NOTE_X5_5);
+		} = generate_mixer_test_setup(DECODED_SUBSTRATE_ADDRESS, DECODED_SUBSTRATE_ADDRESS, MIXER_NOTE_V1_X5_5);
 
 		let proof_input = proof_input_builder.build_js().unwrap();
 		let proof = generate_proof_js(proof_input).unwrap();
@@ -479,12 +602,16 @@ mod test {
 	}
 
 	#[wasm_bindgen_test]
-	fn anchor_proving() {
+	fn anchor_proving_v1() {
 		let AnchorTestSetup {
 			proof_input_builder,
 			vk,
 			..
-		} = generate_anchor_test_setup(DECODED_SUBSTRATE_ADDRESS, DECODED_SUBSTRATE_ADDRESS, ANCHOR_NOTE_X5_4);
+		} = generate_anchor_test_setup(
+			DECODED_SUBSTRATE_ADDRESS,
+			DECODED_SUBSTRATE_ADDRESS,
+			ANCHOR_NOTE_V1_X5_4,
+		);
 
 		let proof_input = proof_input_builder.build_js().unwrap();
 		let proof = generate_proof_js(proof_input).unwrap();
@@ -492,4 +619,64 @@ mod test {
 		let is_valid_proof = verify_unchecked_raw::<Bn254>(&proof.public_inputs, &vk, &proof.proof).unwrap();
 		assert!(is_valid_proof);
 	}
+
+	#[wasm_bindgen_test]
+	fn anchor_proving_v2() {
+		let AnchorTestSetup {
+			proof_input_builder,
+			vk,
+			..
+		} = generate_anchor_test_setup(
+			DECODED_SUBSTRATE_ADDRESS,
+			DECODED_SUBSTRATE_ADDRESS,
+			ANCHOR_NOTE_V2_X5_4,
+		);
+
+		let proof_input = proof_input_builder.build_js().unwrap();
+
+		let proof = generate_proof_js(proof_input.clone()).unwrap();
+
+		// UNCOMENT FOR DEBUGGING
+		// match proof_input.inner {
+		// 	ProofInput::Mixer(_) => (),
+		// 	ProofInput::Anchor(anchor_proof_input) => {
+		// 		wasm_bindgen_test::console_log!("exponentiation {:?}",
+		// anchor_proof_input.exponentiation); 		wasm_bindgen_test::console_log!("width
+		// {:?}", anchor_proof_input.width); 		wasm_bindgen_test::console_log!("curve
+		// {:?}", anchor_proof_input.curve); 		wasm_bindgen_test::console_log!("backend
+		// {:?}", anchor_proof_input.backend); 		wasm_bindgen_test::console_log!("secret
+		// {:?}", anchor_proof_input.secret); 		wasm_bindgen_test::console_log!("nullifier
+		// {:?}", anchor_proof_input.nullifier); 		wasm_bindgen_test::console_log!("
+		// recipient {:?}", anchor_proof_input.recipient); 		wasm_bindgen_test::
+		// console_log!("relayer {:?}", anchor_proof_input.relayer); 		wasm_bindgen_test::
+		// console_log!("refund {:?}", anchor_proof_input.refund); 		wasm_bindgen_test::
+		// console_log!("fee {:?}", anchor_proof_input.fee); 		wasm_bindgen_test::
+		// console_log!("chain id {:?}", anchor_proof_input.chain_id);
+		// 		wasm_bindgen_test::console_log!("leaves {:?}", anchor_proof_input.leaves);
+		// 		wasm_bindgen_test::console_log!("leaf index {:?}",
+		// anchor_proof_input.leaf_index); 		wasm_bindgen_test::console_log!("roots {:?}",
+		// anchor_proof_input.roots);
+
+		// 		let params = setup_params(ArkCurve::Bn254, 5, 4);
+		// 		let poseidon = Poseidon::<Bn254Fr>::new(params);
+
+		// 		let chain_id = Bn254Fr::from(anchor_proof_input.chain_id);
+		// 		let secret = Bn254Fr::from_le_bytes_mod_order(&anchor_proof_input.secret);
+		// 		let nullifier =
+		// Bn254Fr::from_le_bytes_mod_order(&anchor_proof_input.nullifier); 		let res =
+		// poseidon.hash(&[chain_id, nullifier, secret]).unwrap(); 		let res_bytes =
+		// res.into_repr().to_bytes_le(); 		let nullifier_hash =
+		// poseidon.hash_two(&nullifier, &nullifier).unwrap(); 		let nullifier_hash_bytes
+		// = nullifier_hash.into_repr().to_bytes_le(); 		wasm_bindgen_test::console_log!("
+		// calc leaf {:?}", res_bytes); 		wasm_bindgen_test::console_log!("nullifier hash
+		// {:?}", nullifier_hash_bytes); 	},
+		// };
+
+		// for p in &proof.public_inputs {
+		// 	wasm_bindgen_test::console_log!("{:?}", p);
+		// }
+
+		let is_valid_proof = verify_unchecked_raw::<Bn254>(&proof.public_inputs, &vk, &proof.proof).unwrap();
+		assert!(is_valid_proof);
+	}
 }
